## pluginhandler for jabberif
##

from string import split,strip,join
import sys,os
import imp			## module loading
import traceback		## module import + function execution
import jabber
import threading
import time

class pluginhandler:
	def __init__(self, uh, lh, config):
		self.__plugins = {}
		self.__commands = {}
		self.__uh = uh ## userhandler instance
		self.__lh = lh ## loghandler instance
		self.__config = config
		
		self.__threads = {} ## running threads by user
		self.__use_threading = 0
		if config['use_threading'] in ['1', 'on', 'yes']:
			self.__use_threading = 1

		## dos handling
		self.__access_time = {} ## time of last command by user
		self.__min_access_time = float(config['min_access_time'])
		self.__ignore_time = float(config['ignore_time'])
	
	def plugins(self):
		return self.__plugins
	def commands(self):
		return self.__commands

	####################################################################################################
	## plugin module handling

	def load(self, name):
		"""load plugin module"""
		if self.__plugins.has_key(name):
			self.__lh.log("error loading: plugin %s already exists" % name, 'error')
			return 1
			
		if not os.path.isfile(self.__config['plugindir']+'/'+name+'.py'):
			self.__lh.log("error loading: plugin %s doesn't exist" % name, 'error')
			return 1

		fp, pathname, description = imp.find_module(name, [self.__config['plugindir']])
		try:
			module = imp.load_module(name, fp, pathname, description)
			self.__plugins[name]={'name': name, 'module': module,
				'obj': eval('module.'+name+'(self)')}
		except ImportError, e:
			self.__lh.log("error importing module %s: %s" % (name, str(e)), 'error')
			if fp:
				fp.close()
			return 1
		except Exception, e:
			self.unload(name)
			self.__lh.log("error loading module %s: %s" % (name, str(e)), 'error')
			self.__lh.log("--- stack traceback ---\n%s-----------------------" % join(traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback), ''), 'traceback')
			if fp:
				fp.close()
			return 1
		if fp:
			fp.close()
		return 0
	
	def register_command(self, module, name, function, help='', args = {}):
		"""register command
			this is usually called by plugins to register their new commands
			args can be:
				op_only: 0/1"""
		if self.__commands.has_key(name):
			self.__lh.log("error registering cmd: %s already exists in plugin %s" % (name, self.__commands[name]['module'].name), 'error')
		else:
			self.__lh.log("registering command /%s from plugin %s" % (name, module.name), 'module')
			self.__commands[name] = {'name': name, 'module': module, 'function': function,
				'help': help, 'args': args}
				
	def unload(self, name):
		"""unload plugin module"""
		if not self.__plugins.has_key(name):
			self.__lh.log("error unloading: plugin %s is not loaded" % name, 'error')
			return

		module = self.__plugins[name]['obj']

		if hasattr(module, '_unload_') and callable(module._unload_):
			try:
				module._unload_()
			except Exception, e:
				self.__lh.log("error calling _unload_ in plugin %s: %s" % (name, str(e)), 'error')
				self.__lh.log("--- stack traceback ---\n%s-----------------------" % join(traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback), ''), 'traceback')
				

		for cmd in self.__commands.keys():
			if self.__commands[cmd]['module'] is module:
				self.__commands.pop(cmd)
				
		self.__plugins.pop(name)
	
	def unload_all(self):
		for module in self.__plugins.keys():
			self.unload(module)

	####################################################################################################
	## command handling

	def _start_thread(self, fkt, args):
		user = args['msg'].getFrom().getStripped()
		if self.__threads.has_key(user) and str(type(self.__threads[user])) == "<class 'threading.Thread'>" and self.__threads[user].isAlive():
			self.send_back(args['con'], args['msg'], 'hey! you are too fast. i am no finished processing your previous request. please wait...')
			return
		self.__threads[user] = threading.Thread(target=fkt, args=(args,))
		self.__threads[user].start()

	def commandhandler(self, args):
		"""here the commands are relayed to plugin modules"""
		con = args['con']
		msg = args['msg']
		cmd = args['cmd']

		if len(cmd) <= 0:
			return

		## dos handling
		now = time.time()
		user = msg.getFrom().getStripped()
		if self.__access_time.has_key(user):
			if self.__access_time[user].has_key('ignore') and (self.__access_time[user]['ignore'] + self.__access_time[user]['last_access']) > now:
				## ingoring request
				return
			if abs(now - self.__access_time[user]['last_access']) < self.__min_access_time:
				## user is too fast. possible dos attack
				self.__access_time[user] = {'last_access': now, 'ignore': self.__ignore_time}
				self.send_back(con, msg, "dos detected. you are being ignored for %.1f seconds." % self.__ignore_time)
				return
		self.__access_time[user] = {'last_access': now}
		
		## command execution
		if self.__commands.has_key(cmd[0]):
			command = self.__commands[cmd[0]]
			if command['args'].has_key('op_only') and command['args']['op_only'] == 1 and not self.__uh.check_op(msg.getFrom().getStripped()):
				self.send_back(con, msg, 'sorry. you require op status to execute this command')
			else:
				try:
					if self.__use_threading:
						self._start_thread(command['function'], args)
					else:
						command['function'](args)
				except Exception, e:
					self.__lh.log("error executing command %s: %s" % (cmd[0], str(e)), 'error')
					self.__lh.log("--- stack traceback ---\n%s-----------------------" % join(traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback), ''), 'traceback')
					self.send_back(con, msg, 'error executing function')
				
		else:
			self.send_back(con, msg, 'error. try /help')


	####################################################################################################
	## plugin services

	def send(self, con, to, message):
		"""send message"""
		msg = jabber.Message(to, message)
		msg.setType('chat')
		con.send(msg)
		self.__lh.log(str(to) + ': ' + message, 'outmessage')
		
	def send_back(self, con, msg, message):
		"""send message back to where it came from"""
		self.send(con, msg.getFrom(), message)
	
	def get_config(self, key):
		"""get config file entry from section 'plugins'"""
		if self.__config.has_key(key):
			return self.__config[key]
		return None

	def get_userhandler(self):
		"""return userhandler"""
		return self.__uh
		
	def get_loghandler(self):
		"""return loghandler"""
		return self.__lh

	def get_cmd_attr(self, cmdname, attr):
		"""get command attribute"""

		## continue if command exists
		if not self.__commands.has_key(cmdname):
			return None

		## continue if command has attributes
		if not self.__commands[cmdname].has_key('args'):
			return None

		## continue if supplied command attribute exists
		if not self.__commands[cmdname]['args'].has_key(attr):
			return None

		## return attribute
		return self.__commands[cmdname]['args'][attr]
		
