#!/usr/bin/env python
#
# jabber interface.. to whatever you like
#
# remark:
# jabber connection code is based on the console sample jabber client from
# python-jabber
#
#
# version 0.3d - 20040415
# BeF <bef@erlangen.ccc.de>
#

##################################################
## general imports

from string import split,strip,join
import sys,os

import ConfigParser		## config file parsing
import socket			## error handling


########## CONFIG ##########
configfile = os.path.dirname(sys.argv[0]) + '/jabberif.conf'


##################################################
## don't change anything below unless you know what you're doing

##################################################
## config file parsing

conf = {}

def read_config(configfilename = configfile):
	config = ConfigParser.ConfigParser()
	config.set('DEFAULT', 'home', os.path.abspath(os.path.dirname(sys.argv[0])))
	config.read(configfilename)

	for section in config.sections():
		conf[section] = {}
		for name, value in config.items(section):
			conf[section][name] = value

def check_config(continue_on_errors=0):
	wholeconfig = {	'general': {
				'module_path': './contrib/modules',
				'handler_path': './handlers'},
			'account': {
				'server':None, ## None means required
				'username':None,
				'password':None,
				'resource':None,
				'port': '5223',
				'ssl': 'on',
				'priority': '5'},
			'logging': {
				'logfile': '/tmp/jabberif.log',
				'coloroutput': 1},
			'plugins': {
				'plugindir': os.path.dirname(sys.argv[0]) + '/plugins',
				'startup': "['base_plugin']",
				'use_threading':'1',
				'min_access_time': '0.7',
				'ignore_time':'10.0'},
			'users': {
				'ops':None}
			}
	
	error = ''
	had_error = 0
	for section in wholeconfig.keys():
		if not conf.has_key(section):
			error = 'missing section "'+section+'"'
			had_error = 1
			if not continue_on_error:
				return had_error, error
		else:
			for option in wholeconfig[section].keys():
				value = wholeconfig[section][option]
				if value == None:
					if not conf[section].has_key(option):
						error = 'mission option "'+option+'" in section "'+section+'"'
						had_error = 1
						if not continue_on_error:
							return had_error, error
				else:
					if not conf[section].has_key(option):
						conf[section][option] = value
	return had_error, error

read_config()
was_error, errormsg = check_config()
if was_error:
	print "error:", errormsg
	sys.exit(1)
	


##################################################
## additional imports

## module import
module_dirs = os.listdir(conf['general']['module_path'])
if module_dirs != []:
	sys.path += module_dirs
sys.path.append(conf['general']['module_path'])
import jabber

## handler import
sys.path.insert(1, conf['general']['handler_path'])
from userhandler import userhandler
from loghandler import loghandler
from pluginhandler import pluginhandler
from jabberhandler import jabberhandler



##################################################
## cleanup/sessionsave
def cleanup():
	lh.log("end of fnord.", 'info')
	ph.unload_all()

##################################################
## load handlers

## logging
lh = loghandler(conf['logging'])

## load user info
uh = userhandler(conf['users'])

## load plugins
ph = pluginhandler(uh=uh, lh=lh, config=conf['plugins'])

for plugin in eval(conf['plugins']['startup']):
	ph.load(plugin)

## jabber stuff
jh = jabberhandler(config=conf['account'], lh=lh, uh=uh, ph=ph)

##################################################

## connect
jh.connect()

## loop forever
while(1):
	try:
		jh.con.process(1)
	except socket.sslerror:
		cleanup()
		sys.exit(0)
	except Exception, e:
		print "unknown error", str(e)
		cleanup()
		sys.exit(1)
	
	
