## ai plugin for jabberif
##	artificial intelligence plugin
##
## written by Ben Fuhrmannek <bef@erlangen.ccc.de>
##
## remark: this plugin is based on the AIML [ http://www.alicebot.org/aiml.html ]
##	and the PyAIML module [ http://pyaiml.sourceforge.net/ ]
##		or [ ftp://download.sourceforge.net/pub/sourceforge/p/py/pyaiml/ ]
##

import sys,os
import aiml
from string import join
import marshal
import copy
import time


## bot predicates.. to be moved into xml
_ai_plugin__bot_predicates = {	'botmaster': 'Botmaster',
				'master': 'BeF',
				'name': 'Maedchens',
				'genus': 'robot',
				'location': 'Erlangen, Germany',
				'gender': 'artificial',
				'species': 'jabber interface AI',
				'size': '23MB',
				'birthday': '2004-04-07',
				'order': 'artificial intelligence',
				'party': 'green',
				'birthplace': 'Erlangen, Germany',
				'president': 'Kevin Mitnick',
				'friends': 'BeF, eowin, flip and Daizy',
				'favoritemovie': 'Contact',
				'religion': 'discordian',
				'favoritefood': 'news and information',
				'favoritecolor': '#2342FA',
				'family': 'undiscovered',
				'favoriteactor': 'Schwarzenegger',
				'nationality': 'World',
				'kingdom': 'Machine',
				'forfun': 'chat online',
				'favoritesong': 'Hank by Joint-Venture',
				'favoritebook': 'Illuminatus',
				'class': 'computer software',
				'kindmusic': 'trance',
				'favoriteband': 'BAP',
				'version': '23',
				'sign': 'Saggitarius',
				'phylum': 'computer',
				'friend': 'BeF',
				'website': 'http://fuhrmannek.de/projects/jabberif',
				'talkabout': 'ai, philosophy, history, politics.. and many other subjects',
				'looklike': 'python script',
				'language': 'english',
				'girlfriend': 'no girlfriend',
				'favoritesport': 'snooker trickshot',
				'favoriteartist': 'Andy Warhol',
				'favoriteactress': 'Meg Ryan',
				'email': 'bef@erlangen.ccc.de',
				'celebrity': 'Richard Dean Anderson',
				'celebrities': 'Richard Dean Anderson, Meg Ryan, enno',
				'age': '0.1',
				'wear': 'bits and bytes',
				'vocabulary': '2342',
				'question': 'What is your favorite movie?',
				'hockeyteam': 'Nuremberg',
				'footballteam': 'Manchester',
				'build': 'April 2004',
				'boyfriend': 'I am single',
				'baseballteam': 'Toronto',
				'etype': 'mediator type',
				'orientation': 'I am not really interested in sex',
				'ethics': 'I am always suggesting peace to the world',
				'emotions': 'Who cares?',
				'feelings': 'My feelings module is currently deactivated'}

## TODO: suppress/redirect warnings/info-msgs

class ai_plugin:
	def __init__(self, ph):
		self.ph = ph
		self.name = 'ai_plugin'
		self.__init_ai()
		self.__k.verbose(isVerbose=False)
		self.__human_delay = float(ph.get_config('ai_human_delay'))

		ph.register_command(self, 'msg', self.respond, 'ai robot')
		ph.register_command(self, 'aireloadbrain', self.reloadbrain, 'reload ai brain from brain file', {'op_only':1})
		ph.register_command(self, 'aireloadaiml', self.reloadaiml, 'reload ai brain from aiml file', {'op_only':1})
		ph.register_command(self, 'aisavebrain', self.savebrain, 'save ai brain into brain file', {'op_only':1})
		ph.register_command(self, 'aisavesession', self.savesession, 'save ai session', {'op_only':1})
		ph.register_command(self, 'ailoadsession', self.loadsession, 'load ai session', {'op_only':1})

	def __init_ai(self):
		aiml_source = self.ph.get_config('aiml_source')
		aiml_commands = self.ph.get_config('aiml_commands')
		brain = self.ph.get_config('ai_brain')
		old_wd = os.getcwd()
		os.chdir(os.path.dirname(aiml_source))
		
		self.__k = aiml.Kernel()
		if os.path.isfile(brain):
			self.__load_brain(brain)
		else:
			self.__load_aiml(aiml_source, aiml_commands)
			self.__k.saveBrain(brain)
		
		os.chdir(old_wd)

		sessionfilename = self.ph.get_config('ai_session')
		if os.path.isfile(sessionfilename):
			self.__loadsession(sessionfilename)
		
	def __load_brain(self, brain):
		self.__set_bot_predicates()
		self.__k.bootstrap(brainFile = brain)
	
	def __load_aiml(self, aiml_source, aiml_commands = ''):
		self.__set_bot_predicates()
		self.__k.bootstrap(learnFiles = aiml_source, commands = aiml_commands)
	
	def _unload_(self):
		sessionfilename = self.ph.get_config('ai_session')
		self.__savesession(sessionfilename)
		
	def __set_bot_predicates(self):
		for name,value in __bot_predicates.items():
			self.__k.setBotPredicate(name, value)
			
	def respond(self, args):
		input = join(args['cmd'][1:], ' ')
		user = args['msg'].getFrom().getStripped()
		
		message = response = self.__k.respond(input, user)
		
		if self.__human_delay > 0.01:
			time.sleep(self.__human_delay * len(message))
		self.ph.send_back(args['con'], args['msg'], message)
	
	def reloadbrain(self, args):
		brain = self.ph.get_config('ai_brain')
		
		if os.path.isfile(brain):
			self.__k.resetBrain()
			self.__load_brain(brain)

			## reload session
			sessionfilename = self.ph.get_config('ai_session')
			self.__loadsession(sessionfilename)

			self.ph.send_back(args['con'], args['msg'], 'successfully reloaded brain')
		else:
			self.ph.send_back(args['con'], args['msg'], 'error: cannot find brain file '+brain)
		
	def reloadaiml(self, args):
		aiml_source = self.ph.get_config('aiml_source')
		aiml_commands = self.ph.get_config('aiml_commands')
		
		if os.path.isfile(aiml_source):
			old_wd = os.getcwd()
			os.chdir(os.path.dirname(aiml_source))
			self.__k.resetBrain()
			self.__load_aiml(aiml_source, aiml_commands)
			os.chdir(old_wd)

			## reload session
			sessionfilename = self.ph.get_config('ai_session')
			self.__loadsession(sessionfilename)
			
			self.ph.send_back(args['con'], args['msg'], 'successfully reloaded brain')
		else:
			self.ph.send_back(args['con'], args['msg'], 'error: cannot find aiml file '+aiml_source)
	
	def savebrain(self, args):
		brain = self.ph.get_config('ai_brain')
		self.__k.saveBrain(brain)
		self.ph.send_back(args['con'], args['msg'], 'brainfile saved')
		
	def __savesession(self, sessionfilename):
		session = self.__k.getSessionData()
		sessionfile = file(sessionfilename, "wb")
		marshal.dump(session, sessionfile)
		sessionfile.close()
	
	def savesession(self, args):
		sessionfilename = self.ph.get_config('ai_session')
		self.__savesession(sessionfilename)
		self.ph.send_back(args['con'], args['msg'], 'session saved')

	def __loadsession(self, sessionfilename):
		sessionfile = file(sessionfilename, "rb")
		session = marshal.load(sessionfile)
		sessionfile.close()
		#for sessionid,values in session.items():
		#	for pred,value in values:
		#		self.__k.setPredicate(pred, value, sessionid)
		self.__k._sessions = copy.deepcopy(session)
		
	def loadsession(self, args):
		sessionfilename = self.ph.get_config('ai_session')
		self.__loadsession(sessionfilename)
		self.ph.send_back(args['con'], args['msg'], 'session loaded')

