## base plugin
##	offers basic functionality for jabberif
##
## written by Ben Fuhrmannek <bef@erlangen.ccc.de>
##

import sys
from string import join
import jabber

class base_plugin:
	def __init__(self, ph):
		self.ph = ph
		self.name = 'base_plugin'

		ph.register_command(self, 'help', self.help, 'help')
		ph.register_command(self, 'quit', self.quit, 'quit application', {'op_only':1})
		
		ph.register_command(self, 'mload', self.mload, '[mods] load modules', {'op_only':1})
		ph.register_command(self, 'munload', self.munload, '[mods] unload modules', {'op_only':1})
		ph.register_command(self, 'mreload', self.mreload, '[mods] reload modules', {'op_only':1})
		ph.register_command(self, 'mlist', self.mlist, 'list loaded modules')
		
		ph.register_command(self, 'ulist', self.ulist, 'list users', {'op_only':1})
		ph.register_command(self, 'usendpr', self.usendpresence, '<id> <type> send presence', {'op_only':1})


	#### general functions
	def help(self, args):
		shorthelp = []
		allcmds = self.ph.commands().keys()
		allcmds.sort()

		check_op = self.ph.get_userhandler().check_op(args['msg'].getFrom().getStripped())

		for cmdname in allcmds:
			cmd = self.ph.commands()[cmdname]
			op_only = self.ph.get_cmd_attr(cmdname, 'op_only') ## can be None or '0'/'1'
			## this means 'if we have op or no op is needed'
			if check_op or not (op_only and int(op_only) == 1):
				shorthelp.append("/%-10s  %s" % (cmd['name'], cmd['help']))
		self.ph.send_back(args['con'], args['msg'], '\n'+join(shorthelp, '\n'))
		
	def quit(self, args):
		args['con'].disconnect()
		raise Exception('normal /quit')


	#### module functions
	def munload(self, args):
		plugins = self.ph.plugins()
		backmsg = []
		for mod in args['cmd'][1:]:
			if plugins.has_key(mod):
				self.ph.unload(mod)
				backmsg.append('undloaded module '+mod)
			else:
				backmsg.append('there is no module named '+mod)
		self.ph.send_back(args['con'], args['msg'], '\n'+join(backmsg, '\n'))
	
	def mload(self, args):
		plugins = self.ph.plugins()
		backmsg = []
		for mod in args['cmd'][1:]:
			if plugins.has_key(mod):
				backmsg.append('module '+mod+' is already loaded')
			else:
				if self.ph.load(mod) == 0:
					backmsg.append('loaded module '+mod)
				else:
					backmsg.append('error loading module '+mod)
		self.ph.send_back(args['con'], args['msg'], '\n'+join(backmsg, '\n'))
	
	def mreload(self, args):
		plugins = self.ph.plugins()
		backmsg = []
		for mod in args['cmd'][1:]:
			if plugins.has_key(mod):
				self.ph.unload(mod)
				self.ph.load(mod)
				backmsg.append('re-loaded module '+mod)
			else:
				if self.ph.load(mod) == 0:
					backmsg.append('loaded module '+mod)
				else:
					backmsg.append('error loading module '+mod)
		self.ph.send_back(args['con'], args['msg'], '\n'+join(backmsg, '\n'))
	
	def mlist(self, args):
		plugins = self.ph.plugins()
		pnames = plugins.keys()
		pnames.sort()
		self.ph.send_back(args['con'], args['msg'], '\n'+join(pnames, '\n'))
			

	#### user functions
	
	def ulist(self, args):
		uh = self.ph.get_userhandler()
		outs = []
		for user in uh.get_all_users():
			attrs = []
			for attrib in ['op']: ## all listed attributes
				attr = uh.check_attr(user, attrib)
				if attr:
					attrs.append(attrib+'='+str(attr))
			outs.append(user+' ['+join(attrs, ',')+']')
			presence = uh.get_presence(user)
			for pu in presence.keys():
				outs.append(pu+': '+presence[pu])
		self.ph.send_back(args['con'], args['msg'], '\n'+join(outs, '\n'))
	
	def usendpresence(self, args):
		uh = self.ph.get_userhandler()
		data = args['cmd'][1:]
		if len(data) != 2:
			self.ph.send_back(args['con'], args['msg'], 'syntax error. see /help')
			return
		id, type = data
		args['con'].send(jabber.Presence(to=id, type=type))
		

## test -- not functional at the moment
if __name__ == '__main__':
	class p:
		def __init__(self):
		        self.__plugins = []
			self.__commands = {}
						
		def register_command(self, module, name, function, help=''):
			pass
		def send_back(self, con, msg, message):
			pass
	
	ph = p()
	
	bp = base_plugin(ph)
